#!/usr/bin/python
from __future__ import print_function

'''
    TSP_Utilities.py

    Utilities for solving the Traveling Salesman Problem.

    Author: Ottmar Kraemer-Fuhrmann <ottmar.kraemer-fuhrmann@scai.fraunhofer.de>

    Adapted to python version 2.x with x>5 
    by 
    Anton Schueller <anton.schueller@scai.fraunhofer.de>


    $Id$

    Copyright (C) 2012, Fraunhofer SCAI, Germany
    All rights reserved
'''

#################
# read_distance #
#################
def read_distance(filename,size):
    ''' Read a symmetric distance distance from an external file.
        The input file contains the upper triangle distances as integers (without main diagonal).
        The main diagonal is set to 0.

        Input:    filename Name of the input file to be read.
        Output:   Quadratic integer distance matrix.
    '''

    # Open the file and read the first line to get the problem dimension.
    inFile = open(filename, "r")
    token = inFile.readline().split()
    #size = len(token) + 1

    # Create the distance matrix.
    distance = [[0 for j in range(size)] for i in range(size)]

    # Set first row and column.
    distance[0][0] = 0
    for i in range(1, size):
        distance[0][i] = int(token[i - 1])
        distance[i][0] = distance[0][i]

    # Read all other lines.
    for i in range(1, size):
        token = inFile.readline().split()
        distance[i][i] = 0
        for j in range(i + 1, size):
            distance[i][j] = int(token[j - i - 1])
            distance[j][i] = distance[i][j]
    return distance

##################
# print_distance #
##################
def print_distance(distance):
    ''' Print a quadratic distance distance matrix.

        Input:  distance Quadratic distance matrix.
    '''
    print("\nDistances:")
    size = len(distance)
    for i in range(size):
        print(repr(i).rjust(3), ")", end=" ")
        for j in range(size):
            print(repr(distance[i][j]).rjust(4), end=" ")
        print()
        
###############
# tour_length #
###############
def tour_length(distance, tour):
    ''' Compute the length of a given tour.

        Input:  distance Quadratic distance matrix.
                tour List of cities forming the tour.
        Output: Tour length.
    '''
    size = len(distance)
    length = distance[tour[0]][tour[size - 1]]
    for i in range(0, size - 1):
        length += distance[tour[i]][tour[i + 1]]
    return length
